from os import listdir
from subprocess import check_output, CalledProcessError, STDOUT
import traceback
import sys


class System:

    def __init__(self, sys_class_net="/sys/class/net"):
        self.sys_class_net = sys_class_net

    def disk_is_ready(self, disk):
        return self.run(['blkid', disk])

    def disks_available(self):
        return check_output(['blkid', '-o', 'device']).splitlines()

    def mount_disk(self, disk, directory):
        return self.run(['mount', disk, directory])

    def fix_permissions(self, directory, user):
        chmod = ['chmod', '770', directory]
        chown = ['chown', '-R', '{}:{}'.format(user, user), directory]
        return self.run(chmod) and self.run(chown)

    def network_is_up(self, name):
        try:
            with open("{}/{}/operstate".format(self.sys_class_net, name), 'r') as f:
                status = f.read().strip()
                print "{}: {}".format(name, status)
                return status == 'up'
        except Exception as e:
            print 'error: args: {}, message: {}'.format(e.args, e.message)
            return False

    def networks_available(self):
        try:
            return listdir(self.sys_class_net)
        except Exception as e:
            print 'error: args: {}, message: {}'.format(e.args, e.message)
            return []

    def sync_time(self):
        return self.run(['ntpdate', '-u', 'pool.ntp.org'])

    def run(self, *args):
        try:
            print("running: '{}'".format(self.srepr(*args)))
            output = check_output(*args, stderr=STDOUT).strip()
            if output:
                print(output)
            return True
        except CalledProcessError as e:
            if e.output.strip():
                print(e.output.strip())
            print 'error: args: {}, message: {}, exit code: {}'.format(e.args, e.message, e.returncode)
        except Exception as e:
            print 'error: args: {}, message: {}'.format(e.args, e.message)

        return False

    def srepr(self, list):
        if isinstance(list, basestring):
            return list
        return " ".join(list)